/***************************************************************************
                          Atom.h  -  description
                             -------------------
    begin                : mer jun 25 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ATOM_H
#define ATOM_H

#include <SimAtom.h>
#include "Accelerometer.h"
#include "Clock.h"
#include "Girometer.h"
#include "Leg.h"
#include "Molecule.h"
#include "CartesianPosition.h"
#include "nommgr.h"
#include <vector>
#include "mmgr.h"


namespace API {
  class Leg;
  class Molecule;
  class Accelerometer;
  class Girometer;
  class Clock;

/**
* Classe definissant un atome
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 25/05/03 (25 mai 2003)
*@since 0.1
*/

  class Atom {

    class Util;
    friend class Util;
    friend class Molecule;

    private:
      int ID;
      Molecule * molecule;
      Clock * clock;
      Leg * leg[6];
      Girometer * giro[3];
      Accelerometer * accel[3];

    public:

/**
* Classe utilitaire completant la definition d'un atome dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 25/09/03 (25 septembre 2003)
*@since 0.1
*/

      class Util {
        friend class Atom;
        private:
          Atom * atom;
          SimAtom * simatom;
          Util(Atom * anAtom, SimAtom * aSimAtom);
          ~Util();          
        public:

/**
	*
	* retourne l'instance de la representation interne a la simulation de l'atome
	* @return la reference de l'atome simule
	*/
          SimAtom * getSimulatedObject() const;

/**
	*
	* retourne la position cartesienne du centre de l'atome
	* @return position cartesienne du centre de l'atome
	*/
          CartesianPosition getCartesianPosition() const;

/**
	* affecte une position cartesienne a l'atome courant
	* @param aCartesianPosition reference de la position cartesienne
	*/
          void setCartesianPosition(CartesianPosition * aCartesianPosition);

/**
	* retourne le mode d'attachement a l'environnement de l'atome
	* @return si l'atome est attache a l'environnement
	*/
      bool getAttachToEnvMode() const;
/**
	* affecte le mode d'attachement de l'atome
	* @param onoff mode d'attachement
	*/
      void setAttachToEnvMode(bool onoff);
          
/**
	* affiche l'etat de l'atome courant
	*/
          void print() const;

      };

/**
	* Reference de l'instance de la classe specifique a la simulation d'un atome
	*
	*/            
      Util * util;

/**
	* Constructeur d'un atome
	*
	*/
      Atom();

/**
	* Destructeur d'un atome
	*
	*/
      ~Atom();

	/**
	* Retourne l'identifiant de l'atome courant
	*
	* @return le numero de l'atome
  	*/      
      unsigned int getID() const;
      
  /**
	* Retourne le nombre de pattes de l'atome courant
	*
	* @return Le nombre de pattes de l'atome (normalement 6)
  	*/
      unsigned int getLegCount() const;

  /**
	* Retourne la patte associee au numero physique passe en parametre
	* @param aLegID Identifiant physique de la patte
	*
	* @return la patte associee au numero physique
  	*/
      Leg * getLeg(unsigned int aLegID) const;

  /**
	* Retourne la patte associee au numero logique passe en parametre
	* @param aLegID Identifiant logique de la patte
	*
	* @return La patte associee au numero logique passee en parametre
	*/
      Leg * getMappedLeg(unsigned int aLegID) const;

	/**
	* Retourne la reference de la molecule de l'atome courant
	*
	* @return La reference de l'instance de la molecule qui possede l'atome courant
  	*/
	    Molecule * getMolecule() const;

	/**
	* Renvoie un tableau (un vecteur) indiquant quelle patte logique correspond a quelle patte physique
	* ex : si a la ligne 2 on a la valeur 3, la patte physique 2 correspond a la patte
	* logique 3
	*
	* @return La correspondance patte physique/patte logique de l'atome courant
	*/
      std::vector<unsigned int> getMap() const;

	/**
	* Retourne la reference de l'accelerometre donne en parametre de l'atome courant
	* @param anAccelerometerID Numero identifiant l'accelerometre
  *
	* @return La reference de l'une des instances d'accelerometre de l'atome courant
  	*/
      Accelerometer * getAccelerometer(unsigned int anAccelerometerID) const;

  /**
	* Retourne le nombre d'accelerometre de l'atome courant
	*
	* @return Le nombre d'accelerometre de l'atome (normalement 3)
  	*/
      unsigned int getAccelerometerCount() const;

	/**
	* Retourne la reference de l'horloge de l'atome courant
  *
	* @return La reference de l'instance de l'horloge de l'atome courant
  	*/
      Clock * getClock() const;

	/**
	* Retourne la reference du girometre donne en parametre de l'atome courant
	* @param aGirometerID Numero identifiant le girometre
  *
	* @return La reference de l'une des instances de gyrometre de l'atome courant
  	*/
      Girometer* getGirometer(unsigned int aGirometerID) const;

  /**
	* Retourne le nombre de girometre de l'atome courant
	*
	* @return Le nombre de gyrometre de l'atome (normalement 3)
  	*/
      unsigned int getGirometerCount() const;

	/**
	* Repositionne le mappage des pattes par rapport aux nouvelles 1ere et 2eme pattes
	* passees en parametre
	* @param face1 Nouvelle patte numero 0 (logique )
	* @param face2 Nouvelle patte numero 1 (logique )
	*/
      void setMap(unsigned int face1, unsigned int face2);

	/**
	* Procede a l'arret des mouvements de toutes les pattes de l'atome courant
	*/
      void stop();

	/**
	* Retourne l'atome connecte a la patte physique passee en parametre
	* @param LegID Identifiant physique de la patte
	*
	* @return L'atome connecte a la patte physique passee en parametre
	*/
      Atom *  getNeighbor(unsigned int LegID) const; // index in 0..5

  /**
	* Retourne le nombre de pattes connectees de l'atome courant
	*
	* @return Le nombre de pattes connectees de l'atome (au plus 6)
  	*/
      unsigned int getConnectedLegCount() const;

  /**
	* Retourne le nombre d'atomes distincts connectes a l'atome courant
	*
	* @return le nombre d'atomes reellement connectes a l'atome (de 0 a 6)
  	*/    
      unsigned int getConnectedAtomCount() const; 

//      std::vector<Atom *> * getNeighbors() const;
//      on rebascule vers un tableau de 6 references ...
//      Leg * getMyLinkedLeg(unsigned int neighborIndex, unsigned int legIndex) const;
//      unsigned int getLinkedLegCount(unsigned int neighborIndex) const; // 1..3
    protected:
/**
	* Affecte l'identifiant de l'atome courant
	* @param anAtomID Numero de l'atome
	*/

	    void setID(unsigned int anAtomID);
/**
	* Affecte l'atome courant a une molecule
	* @param aMolecule Reference de la molecule
	*/
      void setMolecule(Molecule * aMolecule);
  };
}
#endif
