/***************************************************************************
                          Molecule.h  -  description
                             -------------------
    begin                : ven jun 08 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Modele statique
/*
  * changes :
  *  - 08/06/03 MD Creation !
  */


#ifndef __MOLECULE__
#define __MOLECULE__


// Include files
#include "nommgr.h"
#include <vector>
#include "mmgr.h"
#include "Atom.h"
#include <RigidBodyEntity.h>
#include "SimMolecule.h"
#include "Link.h"

class SimMolecule;

namespace API {
  class Atom;
  class Link;

/**
* Classe definissant une molecule
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 25/05/03 (25 mai 2003)
*@since 0.1
*/       
  class Molecule
  {
    class Util;
    friend class Util;
    friend class Atom;
    friend class Leg;

    public:

/**
* Classe utilitaire completant la definition d'une molecule dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 25/05/03 (25 mai 2003)
*@since 0.2
*/
      class Util {
        friend class Molecule;
        private:
          Molecule * molecule;
          SimMolecule * simmolecule;

          Util(Molecule * molecule);
          ~Util();
        public:
/**
	* affiche la liste des atomes (sommets) de la molecule courante
	*/
          void          printAtoms() const;

/**
	* affiche la listes des liens (arcs) de la molecule courante
	*/
          void          printLinks() const;
/**
	* affiche la matrice d'adjascence de la molecule courante
	*/
          void          printAdjacencyMatrix() const;
/**
	* affiche les elements connexes de la molecule courante
	*/
          void          printConnectedComponentID() const;

/**
	* affiche l'etat de la molecule courante
	*/
          void          print() const;

/**
	*
	* retourne l'instance de la representation interne a la simulation de la molecule
	* @return la reference de la molecule simulee
	*/
          SimMolecule * getSimulatedObject();

      };

/**
	* Reference de l'instance de la classe specifique a la simulation d'une molecule
	*
	*/
      Util * util;
    
	/**
	* retourne l'identifiant de la molecule courante
	*
	* @return le numero de l'atome
  	*/
      unsigned int  getID();

  /**
	* retourne le nombre d'atomes de la molecule courante
	*
	* @return le nombre d'atomes
  	*/
      unsigned int  getAtomCount() const;

  /**
	* retourne un atome de la molecule courante
	* @param index index de l'atome dans la molecule
  *
	* @return la reference de l'atome
  	*/
      Atom *        getAtom(unsigned int index) const;

  /**
	* retourne l'index d'un atome dans la molecule courante.
  *
  * retourne l'index (la position relative) d'un atome dans la molecule courante
  * @param anAtom la reference de l'atome 
  *
	* @return la position relative de l'atome dans la molecule (0..getAtomCount()-1 ou -1 si l'atome n'appartient pas a la molecule
  	*/
      int           getAtomIndex (Atom * anAtom) const;

  /**
	* retourne le nombre de connexions de la molecule courante.
  *
  * On compte chaque paires de pattes connectees
	*
	* @return le nombre de connexions (0 ou plus)
  	*/
      unsigned int  getLinkCount() const;

  /**
	* retourne le nombre de connexions entre deux atomes de la molecule courante.
  *
  * On compte le nombre de connexions entre deux atomes
  * @param atomIndex1 la position relative (=index) du premier atome
  * @param atomIndex2 la position relative (=index) du deuxieme atome
  *
	* @return le nombre de connexions (0 .. 3, 0 si atomIndex1==atomIndex2)
  	*/      
      unsigned int  getLinkCount(unsigned int atomIndex1,unsigned int atomIndex2) const;

  /**
	* retourne la reference de la (premiere) connexion entre deux atomes de la molecule courante.
  *
  * @param atomIndex1 la position relative (=index) du premier atome
  * @param atomIndex2 la position relative (=index) du deuxieme atome
  * @param linkIndex la position relative (=index) de la connexion
  *
	* @return la reference de la connexion
  	*/      
      Link *        getLink(unsigned int atomIndex1,unsigned int atomIndex2, unsigned linkIndex=0) const;


  /**
	* retourne une connexion de la molecule courante
	* @param index index de la connexion dans la molecule
  *
	* @return la reference de la connexion
  	*/
      Link *        getLink(unsigned int index) const;

  /**
	* retourne l'index d'une connexion dans la molecule courante.
  *
  * retourne l'index (la position relative) d'une connexion dans la molecule courante
  * @param aLink la reference de la connexion
  *
	* @return la position relative de la connexion dans la molecule (0..getLinkCount()-1 ou -1 si la connexion n'appartient pas a la molecule
 	*/
      int           getLinkIndex (Link * aLink) const;

  /**
	* procede a la deconnexion de toutes les pattes de chaque atome de la molecule
  *
  * Cette methode reduit la molecule a un seul atome.
  * De plus il y a creation de getAtomCount()-1 molecules toutes reduites a un seul atome.
  */
      void          atomize();
	/**
	* procede a l'arret des mouvements de toutes les pattes de chaque atome de la molecule
	*/

      void          stop();
	/**
	* donne le prochain index de synchronisation de la molecule
  *
	* @return l'index de synchronisation disponible 
	*/

      unsigned int  getNextSyncIndex();

	/**
	* donne le nombre d'index de synchronisation utilise
  *
	* @return le nombre d'index de synchronisation
	*/

      unsigned int  getSyncIndexCount();

	/**
	* donne l'index de synchronisation actif
  *
  * @param index le rang dans la liste des index de synchronisation
  *
	* @return l'index de synchronisation actif
	*/

      unsigned int  getSyncIndex(unsigned int index);
            
  /**
	* libere l'index de synchronisation la molecule courante.
  *
  * lorsque l'on utilise plus l'ensemble de synchronisation, il faut liberer les ressources
  * @param index l'index de synchronisation
 	*/

      void          releaseSyncIndex(unsigned int index);

  /**
	* renvoie si toutes les entites synchronisees selon l'index de synchronisation ont terminees leur traitement.
  *
  * @param index l'index de synchronisation
  *
	* @return si les traitements sont termines
 	*/
      bool          isFinished(unsigned int index);

    protected:
/**
	* constructeur d'une molecule
	*
	*/
      Molecule ();

/**
	* destructeur d'une molecule
	*
	*/
      ~Molecule();
            
/**
	* affecte l'identifiant de la molecule courante
	* @param aMoleculeID numero de la molecule
	*/
      void          setID      (int aMoleculeID);

/**
	* ajoute un atome a la molecule courante
	* @param atom reference de l'atome a ajouter
	*/
      void          addAtom    (Atom *   atom);

/**
	* enleve un atome de la molecule courante
	* @param atom reference de l'atome a enlever
	*/
      void          removeAtom (Atom *   atom);

/**
	* ajoute une connexion a la molecule courante
	* @param link reference de la connexion a ajouter
	*/
      void          insertLink    (Link*   link);

/**
	* enleve une connexion de la molecule courante
	* @param link reference de la connexion a enlever
	*/
      void          removeLink (Link*   link);

private :
      void          computeConnectedComponent();
      void          computeConnectedComponent(unsigned int atomIndex);
      void          setBaseConnectedComponent(unsigned int atomIndex);
      unsigned int  getConnectedComponentCount() const;
      bool          isConnected(unsigned int atomIndex, unsigned int anotherAtomIndex) const;
      unsigned int  getConnectedComponentCard(unsigned int index) const;
      void          adjItInit(unsigned int atomIndex) ;
      int           adjItBeg() ;
      int           adjItNext() ;

      bool          adjItEnd() const;
		  std::vector<Atom *> atoms;
		  unsigned int atomCount;
		  std::vector<std::vector <Link *> > adj;
		  unsigned int linkCount;
      std::vector<int> connectedComponentID;
      std::vector<unsigned int> syncIndexes;
      int connectedComponentCount;
      int ID;
      int adjItPos;
      int adjItAtom;
  };// END CLASS DEFINITION Molecule
}

#endif // __MOLECULE__
