/***************************************************************************
                          Leg.h  -  description
                             -------------------
    begin                : mer jun 08 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Modele statique
/*
  * changes :
  *  - 08/06/03 MD Creation !
  */


#ifndef __LEG__
#define __LEG__


// Include files
#include "nommgr.h"
#include <string>
#include "mmgr.h"

#include <SimLeg.h>
#include <RigidBodyEntity.h>
#include <Node.h>
#include "Atom.h"
#include "CartesianPosition.h"
#include "ForceSensor.h"
#include "IRReceiver.h"
#include "IRSender.h"
#include "ServoMotor.h"
#include "SphericPosition.h"
#include "TouchSensor.h"
#include "Link.h"
class SimLeg;

namespace API {
  
  class ServoMotor;
  class Atom;
  class ForceSensor;
  class TouchSensor;
  class IRReceiver;
/**
* Classe definissant une patte
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/05/03 (08 juin 2003)
*@since 0.1
*/
  class Leg
  {
    class Util;
    friend class Util;
    friend class Atom;
    friend class Link;    
    private:
      //SimLeg * simleg;
      ServoMotor * lservo[2];
	    IRSender * lirsend;
	    IRReceiver * lirreceiver;
	    ForceSensor * lforce[3];

	    TouchSensor * ltouch;
      int ID;
      Atom * atom;
      int MappedID;
      Link * linkOfLeg ;
    public:
/**
* Classe utilitaire completant la definition d'une patte dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/05/03 (08 juin 2003)
*@since 0.2
*/
      class Util {
        friend class Leg;
        friend class Atom;
        private:
          Leg * leg;
          SimLeg * simleg;
          Util(Leg * aLeg);
          ~Util();
        public:

/**
	* Procede au deplacement de la patte selon les deux degres de liberte
  *
  * Une trajectoire est imposee.
  * Cette methode permet de faire bouger une patte sans piloter les servo-moteurs.
  * @param aSphericPosition La position spherique cible du mouvement
	*/
          void setSphericPosition(SphericPosition aSphericPosition);

/**
	* Procede a une connexion active entre la patte courante et une autre patte
  *

  * La connexion n'a lieu que si les deux pattes sont connectables.  
  * @param aLeg Reference de la patte qui est candidate a la connexion avec la patte courante
	*/
          void link(Leg * aLeg);

/**
	* Retourne l'instance de la representation interne a la simulation de la patte
	* @return La reference de la patte simulee
	*/
          SimLeg * getSimulatedObject() const;

/**
	* Retourne si notre patte est connectable avec une autre
	* @param aLeg Reference de la patte qui est candidate a la connexion avec la patte courante
  * @return Si la patte courante est en position d'etre connecetee avec une autre patte
	*/
          bool isConnectable(Leg * aLeg) const;

/**
	* Retourne si notre patte est sur le sol
  * @return Si la patte courante est en appui sur le sol (Vaut faux si l'atome est en appui sur un autre atome)
	*/
          bool isOnGround() const;
          
/**
	* Affiche l'etat de la patte courante
	*/
          void print() const;

      };

/**
	* Reference de l'instance de la classe specifique a la simulation d'une patte
	*
	*/
      Util * util;
protected :

/**
	* Constructeur d'une patte
	* @param anAtom Reference de l'atome proprietaire de la patte
	* @param LegID Identifiant de la patte (0..5)
  *
	*/
      Leg(Atom * anAtom, int LegID);

/**
	* Destructeur d'une patte
	*
	*/
      virtual ~Leg();
public:

  /**
	* Retourne l'identifiant de la patte courante
	*
	* @return Le numero physique de la patte
  	*/
      int getID() const;
      
  /**
	* Retourne le numero logique de la patte courante
	*
	* @return Le numero logique de la patte
  	*/
	    int getMappedID() const;

  /**
	* Retourne la reference de l'atome de la patte courante
	*
	* @return L'instance de l'atome proprietaire de la patte
  	*/
      Atom * getAtom() const;
      
  /**
	* Retourne la position spherique de la patte courante
	*
	* @return La position spherique de la patte
  	*/
      SphericPosition& getSphericPosition() const;

  /**
	* Retourne la position cartesienne de l'extremite de la patte courante par rapport au centre de l'atome
	*
	* @return La position relative de l'extremite (accroche) de la patte
  	*/      
	    CartesianPosition& getRelCartesianPosition() const;

  /**
	* Retourne la reference de la patte liee a la patte courante
	*
	* @return L'instnce de la patte liee a la patte
  	*/
      Leg * getLinkedLeg();

  /**
	* Retourne la reference de la connexion a laquelle participe la patte courante
	*
	* @return L'instance de la connexion concernee par la patte
  	*/      
      Link * getLink() const;

  /**
	* Retourne le premier message disponible de la patte courante
	*
	* @return Le premier message disponible de la patte
  	*/
      std::string  getMessage();
	/**
	* Retourne la reference du servomoteur donne en parametre de la patte courante
	* @param aServoMotorID Numero identifiant le servo moteur (0 ou 1)
  *
	* @return La reference de l'une des instances de servo moteur de la patte courante
  	*/
      ServoMotor * getServoMotor(unsigned int aServoMotorID) const;

	/**
	* Retourne la reference du capteur d'efforts donne en parametre de la patte courante
	* @param aForceSensorID numero identifiant le capteur d'efforts (0..2)
  *
	* @return La reference de l'une des instances de capteur d'effort de la patte courante
  	*/
      ForceSensor * getForceSensor(unsigned int aForceSensorID) const;

	/**
	* Retourne la reference du capteur de contact de la patte courante
  *
	* @return La reference de l'instance du capteur de contact de la patte courante
  	*/
      TouchSensor * getTouchSensor() const;
      
	/**
	* Retourne la reference du recepteur IR de la patte courante
  *
	* @return La reference de l'instance de recepteur Infra Rouge de la patte courante
  	*/
	    IRReceiver * getIRReceiver() const;
      
	/**
	* Retourne la reference de l'emetteur IR de la patte courante
  *
	* @return La reference de l'instance d'emetteur Infra Rouge de la patte courante
  	*/
	    IRSender * getIRSender() const;

  /**
	* Retourne si la patte est connectee
	*
	* @return s'il y a une connexion, la valeur est vrai.
  	*/
      bool isLinked() const;
      
	/**
	* Prise en compte d'une connexion passive d'une patte passee en parametre avec la patte courante
	* @param aLeg Reference de la patte a connecter a la patte courante
  *
  	*/
	    void setLink(Leg * aLeg);

	/**
	* Envoi d'un message passe en parametre a la patte connectee a la patte courante
	* @param aMessage Message a envoyer
  *
  	*/
      
      void sendMessage(std::string aMessage);
      
	/**
	* Procede a la deconnexion passive de la patte courante
  *
  	*/
	    void unlink();

	/**
	* Procede a l'arret des mouvements de la patte courante
	*/      
      void stop();

 /**
	* Retourne le nombre de servo moteurs de la patte courante
	*
	* @return Le nombre de servo moteurs de la patte (normalement 2)
  	*/
      unsigned int getServoMotorCount() const;

  /**
	* Retourne le nombre de capteurs d'efforts de la patte courante
	*
	* @return Le nombre de capteur d'efforts de la patte (normalement 3)
  	*/            
      unsigned int getForceSensorCount() const;

    protected:
/**
	* affecte l'identifiant de la patte courante
	* @param aLegID Numero de la patte (0..5)
	*/
	    void setID(int aLegID);

/**
	* affecte le numero logique de la patte courante
	* @param aMappedID Numero de la patte logique (0..5)
	*/
      void setMappedID(int aMappedID);

/**
	* affecte l'atome proprietaire de la patte courante
	* @param anAtom Reference de l'atome proprietaire
	*/
	    void setAtom(Atom * anAtom);

  };// END CLASS DEFINITION Leg
}
#endif // __LEG__
