/***************************************************************************
                          ServoMotor.h  -  description
                             -------------------
    begin                : ven jun 08 2003
    copyright            : (C) 2003 by Michel Dubois, Yann Le Guyadec
    email                : Michel.Dubois@univ-ubs.fr, Yann.Le-Guyadec@univ-ubs.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// Modle statique
/*
  * changes :
  *  - 08/06/03 MD Creation !
  */


#ifndef __SERVOMOTOR__
#define __SERVOMOTOR__


// Include files
#include "Leg.h"
#include "SimServoMotor.h"

#define KEEP           0
#define REPLACE        1

class SimServoMotor;

namespace API {

  class Leg;
/**
* Classe definissant un servo moteur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 08/06/03 (08 juin 2003)
*@since 0.2
*/
  class ServoMotor
  {
    friend class Leg;
    friend class Atom;
    class Util;
    friend class Util;

    public:
/**
* Classe utilitaire completant la definition d'un servo moteur dans le simulateur
*
*@author Michel Dubois, Yann Le Guyadec
*@version du 27/09/03 (27 septembre 2003)
*@since 0.2
*/

      class Util {
        friend class ServoMotor;
        friend class Leg;
        friend class Atom;
        private:
          SimServoMotor * simservo;
          ServoMotor * servo;
          Util(ServoMotor * servo);
          ~Util();
        public:
/**
	* affiche l'etat du servo-moteur courant
	*/
          void print() const;
          
/**
	* retourne l'instance de la representation interne a la simulation du servo-moteur
	* @return la reference du servo-moteur simule
	*/
          SimServoMotor * getSimulatedObject() const;

        protected:
/**
	* Affecte l'instance du servo moteur a la classe utilitaire
  * @param aSimServoMotor Reference du servo-moteur a attribuer
	*/

          void setSimulatedObject(SimServoMotor * aSimServoMotor );
      };

/**
	* Reference de l'instance de la classe specifique a la simulation d'un servo-moteur
	*
	*/
      Util * util;
      private:
      Leg * leg;
      int servoID;
      int targetPos;
      int targetVelocity;
    
    public:
	/**
	* retourne l'identifiant du servo-moteur courant
	*
	* @return le numero de l'atome
  	*/
      int     getID() const;

  /**
	* retourne la reference de la patte du servo-moteur courant
	*
	* @return l'instance de la patte qui est concernee par le servo-moteur
  	*/
      Leg *   getLeg() const;
      
  /**
	* retourne la position cible du servo-moteur courant
	*
	* @return la position cible du servo-moteur (getMinPosition()..getMaxPosition())
  	*/
	    int     getTargetPosition() const;  // [SERVO_MOTOR_MIN_POS..SERVO_MOTOR_MAX_POS]

  /**
	* retourne l'angle cible du servo-moteur courant
	*
	* @return l'angle cible du servo-moteur (en radians, getLowStopAngle()..getHighStopAngle())
  	*/
	    double  getTargetAngle() const;

  /**
	* retourne la vitesse cible du servo-moteur courant
	*
	* @return la vitesse cible du servo-moteur (en radians/sec)
  	*/
      double  getTargetVelocity() const;  // en radians/sec

  /**
	* retourne la force cible du servo-moteur courant
	*
	* @return la force cible du servo-moteur (en newton)
  	*/
      double  getTargetForce() const;
      
  /**
	* retourne la position actuelle du servo-moteur courant
	*
	* @return la position actuelle du servo-moteur (getMinPosition()..getMaxPosition())
  	*/
      int     getCurrentPosition() const;
      
  /**
	* retourne la vitesse actuelle du servo-moteur courant
	*
	* @return la vitesse actuelle du servo-moteur (en radians/sec)
  	*/
	    double  getCurrentVelocity() const;

  /**
	* retourne la force actuelle du servo-moteur courant
	*
	* @return la force actuelle du servo-moteur (en newton)
  	*/
      double  getCurrentForce() const;
      
  /**
	* retourne l'angle actuel du servo-moteur courant
	*
	* @return l'angle actuel du servo-moteur (getLowStopAngle()..getHighStopAngle())
  	*/
	    double  getCurrentAngle() const;
      
  /**
	* retourne l'angle minimal du servo-moteur courant
	*
	* @return l'angle minimal du servo-moteur (en radians)
  	*/
      double  getHighStopAngle() const;

  /**
	* retourne l'angle maximal du servo-moteur courant
	*
	* @return l'angle maximal du servo-moteur (en radians)
  	*/
 	    double  getLowStopAngle() const;

  /**
	* retourne la position minimale du servo-moteur courant
	*
	* @return la position minimale du servo-moteur (en unite)
  	*/
      int     getMinPosition() const;

  /**
	* retourne la position maximale du servo-moteur courant
	*
	* @return la position maximale du servo-moteur (en unite)
  	*/
      int     getMaxPosition() const;

  /**
	* Ordonne au servo-moteur de reprendre sa position canonique
	*
	* @param mode indique si l'ordre est immediat (REPLACE) ou sequentiel (KEEP)
	* @param deltaTime indique le delai minimal avant le debut de l'execution de l'ordre
	* @param index donne l'index de synchronisation. La valeur 0 indique aucun synchronisation

   */
      void    setToHalfPosition(int mode=KEEP, Real deltaTime=0,unsigned int index=0);

  /**
	* Affecte au servo-moteur un angle cible
	*
  * @param aTargetAngle Angle cible en radians (getLowStopAngle()..getHighStopAngle())
  * @param mode indique si l'ordre est immediat (REPLACE) ou sequentiel (KEEP)
	* @param deltaTime indique le delai minimal avant le debut de l'execution de l'ordre
	* @param index indique l'ensemble de synchronisation de l'appel. [0 .. ]. 0 indique aucun ensemble de synchronisation
   */
      void    setTargetAngle(double aTargetAngle, int mode=KEEP, Real deltaTime=0,unsigned int index=0);

  /**
	* Affecte au servo-moteur une position cible
	*
  * @param aTargetPosition Position cible en unite (getMinPosition()..getMaxPosition())
  * @param mode indique si l'ordre est immediat (REPLACE) ou sequentiel (KEEP)
	* @param deltaTime indique le delai minimal avant le debut de l'execution de l'ordre
	* @param index indique l'ensemble de synchronisation de l'appel. [0 .. ]. 0 indique aucun ensemble de synchronisation
   */
      void    setTargetPosition(int aTargetPosition, int mode=KEEP, Real deltaTime=0,unsigned int index=0);

  /**
	* Affecte au servo-moteur une vitesse cible
	*
  * @param aTargetVelocity Vitesse cible en radians par seconde
  * @param mode indique si l'ordre est immediat (REPLACE) ou sequentiel (KEEP)
	* @param deltaTime indique le delai minimal avant le debut de l'execution de l'ordre
	* @param index indique l'ensemble de synchronisation de l'appel. [0 .. ]. 0 indique aucun ensemble de synchronisation
   */
      void    setTargetVelocity(int aTargetVelocity, int mode=KEEP, Real deltaTime=0,unsigned int index=0);

  /**
	* Affecte au servo-moteur une force cible
	*
  * @param aTargetForce Force cible en newton
  * @param mode indique si l'ordre est immediat (REPLACE) ou sequentiel (KEEP)
	* @param deltaTime indique le delai minimal avant le debut de l'execution de l'ordre
	* @param index indique l'ensemble de synchronisation de l'appel. [0 .. ]. 0 indique aucun ensemble de synchronisation
   */
      void    setTargetForce(double aTargetForce, int mode=KEEP, Real deltaTime=0,unsigned int index=0);

	/**
	* procede a l'arret du servo-moteur
	*/
      void    stop();

    protected:
/**
	* constructeur du servo-moteur
	*
	*/
      ServoMotor();
/**
	* destructeur du servo-moteur
	*
	*/
      ~ServoMotor();

/**
	* affecte l'identifiant du servo-moteur courant
	* @param aServoMotorID numero du servo-moteur
	*/
      void    setID(int aServoMotorID);

/**
	* affecte la patte proprietaire du servo-moteur courant
	* @param aLeg reference du servo-moteur
	*/            
      void    setLeg(Leg * aLeg);
    
  };// END CLASS DEFINITION ServoMotor
}
#endif // __SERVOMOTOR__
